using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace SystemFunctionLib.Apis
{
    /// <summary>
    /// Contains APIs in user32.dll file.
    /// </summary>
    internal partial class WinApis
    {
        /// <summary>
        /// Registers a system hot key.
        /// </summary>
        /// <param name="hWnd">Handle of the window.</param>
        /// <param name="id">Hot key identifier.</param>
        /// <param name="fsModifiers">Key modifiers.</param>
        /// <param name="vk">Virtual key code.</param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern bool RegisterHotKey(
            IntPtr hWnd,
            int id,
            uint fsModifiers,
            Keys vk
        );

        /// <summary>
        /// Unregisters a system hot key.
        /// </summary>
        /// <param name="hWnd">Handle of the window.</param>
        /// <param name="id">Hot key identifier.</param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern bool UnregisterHotKey(
            IntPtr hWnd,
            int id
        );

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool SystemParametersInfo(
            SPI uiAction,
            uint uiParam,
            IntPtr pvParam,
            SPIF fWinIni);

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public unsafe static extern bool SystemParametersInfo(
            SPI uiAction,
            uint uiParam,
            void* pvParam,
            SPIF fWinIni);

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public unsafe static extern bool SystemParametersInfo(
            SPI uiAction,
            uint uiParam,
            uint pvParam,
            SPIF fWinIni);

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool SystemParametersInfo(
            SPI uiAction,
            uint uiParam,
            ref int pvParam,
            SPIF fWinIni);

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool SystemParametersInfo(
            SPI uiAction,
            uint uiParam,
            ref bool pvParam,
            SPIF fWinIni);

        [DllImport("user32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        public static extern bool SystemParametersInfo(
            SPI uiAction,
            uint uiParam,
            StringBuilder pvParam,
            SPIF fWinIni);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern uint GetDoubleClickTime();

        [DllImport("user32.dll", CharSet = CharSet.Auto, SetLastError = true)]
        public static extern bool SetDoubleClickTime(
            uint uInterval);

        /// <summary>
        /// The windows message that indicates a hot key was pressed.
        /// </summary>
        public const int WM_HOTKEY = 0x0312;
    }
}
