using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace SystemFunctionLib.SystemEnvironment
{
    /// <summary>
    /// Provides data for the SystemHotKeyListener.SystemHotKeyPressed event.
    /// </summary>
    public class SystemHotKeyPressedEventArgs : EventArgs
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="SystemHotKeyPressedEventArgs"/> class.
        /// </summary>
        /// <param name="hotKey">A <see cref="HotKey"/> instance that specifies which hot key was pressed.</param>
        public SystemHotKeyPressedEventArgs(HotKey hotKey)
        {
            _hotKey = hotKey;
        }

        private HotKey _hotKey = null;
        /// <summary>
        /// Gets the hot key that was just pressed.
        /// </summary>
        public HotKey PressedHotKey
        {
            get { return _hotKey; }
        }

        /// <summary>
        /// Gets the name of the hot key that was just pressed.
        /// </summary>
        public string HotKeyName
        {
            get { return _hotKey.Name; }
        }
    }
}
